Crate double_buffer

source ·
Generic and simple double buffer

This only provides DoubleBuffer<T>, for implementing a double buffer pattern.

DoubleBuffer<T> is not limited to bytes arrays or similar buffers, it can be used with any type that requires modify while reading current state and all the changes look as one atomic operation.

Structs

  • Encapsulates a piece of state that can be modified and we want all outside code to see the edit as a single atomic change.